//
// Copyright 2007, Lowell Boggs Jr.
//
// This file or directory, containing source code for a computer program,
// is Copyrighted by Lowell Boggs, Jr.  987 Regency Drive, Lewisville
// TX (USA), 75067.  You may use, copy, modify, and distribute this
// source file without charge or obligation so long as you agree to
// the following:
//
//  1.  You must indemnify Lowell Boggs against any and all financial
//      obligations caused by its use, misuse, function, or malfunction.
//      Further, you acknowledge that there is no warranty of any kind,
//      whatsoever.
//
//  2.  You agree not to attempt to patent any portion of this original
//      work -- though you may attempt to patent your own extensions to
//      it if you so choose.
//
//  3.  You keep this copyright notice with the file and all copies
//      of the file and do not change it anyway except language translation.
//
// You are responsible for enforcing your own compliance with these
// conditions and may not use this source file if you cannot agree to the
// above terms and conditions.
//
// Warning:  not all files in this directory structure are covered by the
// same copyright.  Some of them are part of the GNU source distribution
// and you must obey the GPL copyright for those files.

#include "myOstream.h"

using namespace std;

myOstream &myOstream::operator<< (unsigned char c)
{
  return (*this) << (unsigned long)(c);
}
myOstream &myOstream::operator<< (unsigned short s)
{
  return (*this) << (unsigned long)(s);
}
myOstream &myOstream::operator<< (unsigned int i)
{
  return (*this) << (unsigned long)(i);
}

myOstream &myOstream::put(char c)
{
  return (*this) << c;
}

myOstream &myOstream::operator<< (unsigned long l)
{
  char buffer[40];

  char *scan = &buffer[39];

  *scan-- = 0;

  size_t width=0;

  if(l == 0)
  {
    *scan-- = '0';
    ++width;
  }
  else
  switch(outputBase)
  {
    case hexOutput:
      {

          while(l)
          {
            unsigned int bottomDigit = l & 0xf;
            l >>= 4;

            if(bottomDigit >= 10)
            {
              bottomDigit -= 10;
              bottomDigit += 'A';
            }
            else
            bottomDigit += '0';

            *scan-- = bottomDigit;
            ++width;

          }

          if(showBase_)
          {
            *scan-- = 'x';
            *scan-- = '0';
            ++width;
            ++width;
          }

        
      }
      break;

    case octalOutput:
      {

          while(l)
          {
            unsigned int bottomDigit = l & 0x7;
            l >>= 3;

            bottomDigit += '0';

            *scan-- = bottomDigit;
            ++width;

          }

          if(showBase_)
          {
            *scan-- = '0';
            ++width;
          }

      }
      break;
    default:
      {
          while(l)
          {
            unsigned int bottomDigit = l % 10;
            l /= 10;

            bottomDigit += '0';

            *scan-- = bottomDigit;
            ++width;

          }

      }
      break;
  }

  ++scan;

  dumpWithPad(scan, width);

  return *this;
}
myOstream &myOstream::operator<< (int i)
{
  if(i < 0)
  {
    i = -i;

    if(minWidth)
      -- minWidth;
  }

  unsigned long tmp = static_cast<unsigned long>(i);
                                                     
  *this << tmp;

  minWidth = 0;
  return *this;
}
myOstream &myOstream::operator<< (long i)
{

  if(i < 0)
  {
    i = -i;

    if(minWidth)
      -- minWidth;
  }
  unsigned long tmp = static_cast<unsigned long>(i);
                                                    
  *this << tmp;


  minWidth = 0;
  return *this;
}
myOstream &myOstream::operator<< (char c)
{
  dumpWithPad(&c, 1);

  return *this;
}

myOstream &myOstream::operator<< (char const *p)
{
  dumpWithPad(p, strlen(p));

  return *this;
}
myOstream &myOstream::operator<< (std::string const &s)
{
  dumpWithPad(s.data(), s.size());

  return *this;
}
myOstream &myOstream::operator<< (float const &f)
{
  double d = f;

  return (*this) << d;
}
myOstream &myOstream::operator<< (double const &d)
{
  
  char buffer[256];

  sprintf(buffer, "%f", d);

  size_t width=0;

  {
    // remove trailing 0's from the string

    char *p = buffer;
    
    while(*p)
    {
      ++p;
      ++width;
    }

    --p;

    while(p != buffer && *p == '0')
    {
      *p = 0;
      --p;
      --width;
    }

    if(*p == '.')
    {
       *p=0;
       --width;
    }

  }

  dumpWithPad(buffer, strlen(buffer));

  return *this;
}

myOstream& myOendl(myOstream &s)
{
  s.outputBuffer_ += '\n';
  s.flush();
  s.minWidth = 0;
  return s;
}

myOstream& myOflush(myOstream &s)
{
  s.flush();
  s.minWidth = 0;
  return s;
}

myOstream& myOdec(myOstream &s)
{
  s.outputBase = myOstream::decimalOutput;
  return s;
}

myOstream& myOhex(myOstream &s)
{
  s.outputBase = myOstream::hexOutput;
  return s;
}

myOstream& myOoct(myOstream &s)
{
  s.outputBase = myOstream::octalOutput;
  return s;
}

myOstream& myOshowbase(myOstream &s)
{
  s.showBase_ = true;
  return s;
}
myOstream& myOnoshowbase(myOstream &s)
{
  s.showBase_ = false;
  return s;
}





void myOstream::flush()
{
  char const *p = outputBuffer_.c_str();
  size_t      l = outputBuffer_.size();
  size_t      bytes_written;

  WriteFile(p, l, &bytes_written);

  num_put_ += l; 

  outputBuffer_.clear();

}

size_t myOstream::tellp()
{
  return num_put_ + outputBuffer_.size();
}

void
myOstream::
dumpWithPad(char const *s, size_t stringLength)
{
  size_t width = stringLength;

  while(stringLength--)
  {
    outputBuffer_ += *s++;
  }

  while(width++ < minWidth)
  {
    outputBuffer_ += padCharacter;
  }

  minWidth = 0;


}








